/* Vars data file utility */

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <vars.h>

int
main(int argc, char *argv[])
{
    int pflag = 0, wflag = 0, errs = 0, binary = 0, help = 0, numargs;
    char *file = NULL;
    vscalar *elt;
    vhash *opts;
    vlist *args;
    void *ptr;
    FILE *fp;

    /* Initialise */
    v_debug_env();

    /* Call extra declaration functions */
#ifdef EXTRA_DECLS
    EXTRA_DECLS;
#endif

    /* Set options */
    v_option_flag('b', "binary", &binary,
                  "Assume input is binary");

    v_option_flag('p', "print", &pflag,
                  "Print ASCII data");

    v_option_flag('w', "write", &wflag,
                  "Write binary data");

    v_option_string('o', "output", "file", &file,
                    "Write output to specified file");

    v_option_flag('h', "help", &help,
                  "This help message");

    /* Parse arguments */
    if ((opts = v_getopts(argc, argv)) == NULL)
        v_die("Type `%s -help' for help", argv[0]);

    if (help) {
        v_usage("Usage: %s [options] [files]", argv[0]);
        return 0;
    }

    if (file != NULL &&(fp = freopen(file, "w", stdout)) == NULL)
        v_die("%s: can't open %s for writing: %s",
              argv[0], file, strerror(errno));

    /* Decompress if required */
    v_open_with("\\.gz$", "r", "gzip -qcd %s");
    v_open_with("\\.Z$",  "r", "uncompress -c %s");

    /* Process arguments */
    args = v_getargs(opts);
    if ((numargs = vl_length(args)) == 0) {
        vl_spush(args, "-");
        numargs++;
    }

    vl_foreach(elt, args) {
        file = vs_sgetref(elt);

        /* Open file */
        if (V_STREQ(file, "-")) {
            fp = stdin;
            file = "stdin";
        } else if ((fp = v_open(file, "r")) == NULL) {
            v_die("%s: can't open %s: %s",
                  argv[0], file, strerror(errno));
        }

        /* Read contents */
        if (binary)
            ptr = v_read(fp);
        else
            ptr = v_thaw(fp);

        /* Close it */
        if (fp != stdin)
            v_close(fp);

        /* Check for errors */
        if (ptr == NULL) {
            if (binary)
                v_warn("%s: not Vars binary data", file);
            else
                v_warn("%s, %s", file, v_thaw_error());
            errs++;
            continue;
        }

        /* Write output */
        if (pflag) {
            if (numargs > 1)
                printf("%s: ", file);
            v_freeze(ptr, stdout);
        } else if (wflag) {
            v_write(ptr, stdout);
        } else {
            printf("%s: %s OK\n", file, binary ? "data" : "syntax");
        }

        /* Clean up */
        v_destroy(ptr);
    }

    return errs != 0;
}
