/*
 * This file is part of the Vars library, copyright (C) Glenn Hutchings
 * 1996-2006.
 *
 * The Vars library comes with ABSOLUTELY NO WARRANTY.  This is free
 * software, and you are welcome to redistribute it under certain
 * conditions; see the file COPYING for details.
 */

/* Compatibility with previous Vars versions */

#ifndef VARS_COMPAT_H
#define VARS_COMPAT_H

#include <vars.h>

#define V_INT     V_TYPE_INT
#define V_FLOAT   V_TYPE_FLOAT
#define V_DOUBLE  V_TYPE_DOUBLE
#define V_STRING  V_TYPE_STRING
#define V_POINTER V_TYPE_POINTER
#define V_UNDEF   V_TYPE_UNDEF
#define V_NULL    V_TYPE_NULL

#define VF_LINEAR        V_INTERP_LINEAR
#define VF_LAGRANGE      V_INTERP_LAGRANGE
#define VF_SPLINE        V_INTERP_SPLINE
#define VF_NEAREST       V_INTERP_NEAREST
#define VF_EXTRAPOLATE   V_INTERP_EXTRAPOLATE

#define V_NOTFOUND V_CMP_NOTFOUND
#define V_SMALLER  V_CMP_SMALLER
#define V_GREATER  V_CMP_GREATER

extern int v_fwrite(void *ptr, FILE *fp);

extern int va_fwrite(varray *var, FILE *fp);
extern int vb_fwrite(vbuffer *var, FILE *fp);
extern int vd_fwrite(vdatabase *var, FILE *fp);
extern int vf_fwrite(vfunc *var, FILE *fp);
extern int vg_fwrite(vgraph *var, FILE *fp);
extern int vh_fwrite(vhash *var, FILE *fp);
extern int vi_fwrite(vimage *var, FILE *fp);
extern int vl_fwrite(vlist *var, FILE *fp);
extern int vm_fwrite(vmatrix *var, FILE *fp);
extern int vp_fwrite(vparser *var, FILE *fp);
extern int vq_fwrite(vqueue *var, FILE *fp);
extern int vr_fwrite(vregex *var, FILE *fp);
extern int vs_fwrite(vscalar *var, FILE *fp);
extern int vv_fwrite(vvector *var, FILE *fp);

extern void *v_fread(FILE *fp);

extern varray *va_fread(FILE *fp);
extern vbuffer *vb_fread(FILE *fp);
extern vdatabase *vd_fread(FILE *fp);
extern vfunc *vf_fread(FILE *fp);
extern vgraph *vg_fread(FILE *fp);
extern vhash *vh_fread(FILE *fp);
extern vimage *vi_fread(FILE *fp);
extern vlist *vl_fread(FILE *fp);
extern vmatrix *vm_fread(FILE *fp);
extern vparser *vp_fread(FILE *fp);
extern vqueue *vq_fread(FILE *fp);
extern vregex *vr_fread(FILE *fp);
extern vscalar *vs_fread(FILE *fp);
extern vvector *vv_fread(FILE *fp);

extern char *vt_name(vtype *t);
extern char *vt_ptrname(void *ptr);

extern vheader *vt_header(vtype *t);

extern void vt_declare(void);

extern void vt_copy_with(vtype *t, void *(*func)());
extern void vt_destroy_with(vtype *t, void (*func)());
extern void vt_freeze_with(vtype *t, int (*func)());
extern void vt_print_with(vtype *t, void (*func)());
extern void vt_read_with(vtype *t, void *(*func)());
extern void vt_thaw_with(vtype *t, void *(*func)());
extern void vt_traverse_with(vtype *t, int (*func)());
extern void vt_write_with(vtype *t, int (*func)());

extern vtype *vt_create(char *name, char *code);
extern vtype *vt_find(char *code);
extern vtype *vt_find_name(char *name);
extern vtype *vt_type(void *ptr);

extern vhash *vh_create_table(unsigned size, int reuse);
extern vhash *vh_environ(void);
extern vhash *vh_getopt(int argc, char *argv[]);
extern vlist *vh_sortkeys(vhash *h, int (*compare)(vscalar **s1, vscalar **s2));
extern vlist *vh_sortvalues(vhash *h, int (*compare)(vscalar **s1, vscalar **s2));

extern vlist *vg_path_list(vgraph *g, char *node1, char *node2);

extern void vl_sort_inplace(vlist *l, int (*compare)(vscalar **s1, vscalar **s2));

extern vlist *vl_glob(char *dir, char *pat);
extern vlist *vl_procinfo(void);
extern vlist *vl_stat(char *file);

#endif
