/* Freeze/thaw tests */

#include "test.h"

#define VSIZE 13

int
main(void)
{
    vhash *h, *thaw;
    vmatrix *m;
    vscalar *s;
    vvector *v;
    varray *a;
    vlist *l;
    vfunc *f;
    FILE *fp;
    int i;

    TEST_START("freeze");

    s = vs_dcreate(3.14159);

    m = vm_create();
    vm_store(m, 0, 0, 1.0);
    vm_store(m, 1, 1, 1.0);
    vm_store(m, 2, 2, 1.0);

    v = vv_create(VSIZE);
    for (i = 0; i < VSIZE; i++)
        vv_store(v, i, v_randprob());

    l = vl_list(V_TYPE_STRING,       "brian",
                V_TYPE_INT,          45,
                V_TYPE_FLOAT,        90.34,
                V_TYPE_DOUBLE,       1e22,
                V_TYPE_NULL);

    f = vf_func(1.0, 1.0,
                2.0, 4.0,
                3.0, 9.0,
                VF_END);

    a = va_array(1.0, V_TYPE_STRING,   "derek",
                 2.2, V_TYPE_INT,      634,
                 4.6, V_TYPE_DOUBLE,   3.4e7,
                 8.1, V_TYPE_POINTER,  m,
                 9.9, V_TYPE_POINTER,  f,
                 VA_END);

    h = vh_hash("STRING",       V_TYPE_STRING,       "fred",
                "INT",          V_TYPE_INT,          42,
                "REAL",         V_TYPE_DOUBLE,       11.7,
                "LIST",         V_TYPE_POINTER,      l,
                "VECTOR",       V_TYPE_POINTER,      v,
                "SCALAR",       V_TYPE_POINTER,      s,
                "FUNC",         V_TYPE_POINTER,      f,
                "ANOTHER LIST", V_TYPE_POINTER,      l,
                "MATRIX",       V_TYPE_POINTER,      m,
                "ARRAY",        V_TYPE_POINTER,      a,
                NULL);

    vv_freeze_format("%.2f");
    vv_freeze_linesize(12);

    if ((fp = fopen(TEST_OUTPUT, "w")) == NULL)
        TEST_FAIL("can't open output file");

    fprintf(fp, "# This is a comment line, which should be ignored.\n\n");
    TEST_NOFAIL("freezing", v_freeze(h, fp));
    fclose(fp);

    TEST("thawing", (thaw = v_thaw_file(TEST_OUTPUT)) != NULL);

    if (thaw == NULL) {
        fprintf(stderr, "Thaw error: %s\n", v_thaw_error());
        TEST_ABORT;
    }

    TEST("thaw value 1", vh_iget(h, "INT") == vh_iget(thaw, "INT"));
    TEST("thaw value 2", vh_iget(h, "REAL") == vh_iget(thaw, "REAL"));
    TEST("thaw value 3", v_type(vh_pget(thaw, "LIST")) == vlist_type);
    TEST("thaw value 4", v_type(vh_pget(thaw, "SCALAR")) == vscalar_type);

    TEST_FINISH;
}
